#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# The autofix suggestion mentioned in the pipeline does not work actually.
# It proposed to replace `xml` with `defusedxml` and it does not have such
# import.
# nosemgrep: splunk.use-defused-xml
from xml.etree import ElementTree as ET
from defusedxml import minidom


def _pretty_print_xml(string: str) -> str:
    """
    Returns a pretty-printed XML as a string.
    """
    return minidom.parseString(string).toprettyxml(indent="    ")


def generate_nav_default_xml(include_inputs: bool, include_dashboard: bool) -> str:
    """
    Generates `default/data/ui/nav/default.xml` file.
    """
    nav = ET.Element("nav")
    if include_inputs:
        ET.SubElement(nav, "view", attrib={"name": "inputs"})
    ET.SubElement(nav, "view", attrib={"name": "configuration", "default": "true"})
    if include_dashboard:
        ET.SubElement(nav, "view", attrib={"name": "dashboard"})
    ET.SubElement(nav, "view", attrib={"name": "search"})
    nav_as_string = ET.tostring(nav, encoding="unicode")
    return _pretty_print_xml(nav_as_string)


def generate_views_inputs_xml(addon_name: str) -> str:
    """
    Generates `default/data/ui/views/inputs.xml` file.
    """
    view = ET.Element(
        "view",
        attrib={
            "template": f"{addon_name}:/templates/base.html",
            "type": "html",
            "isDashboard": "False",
        },
    )
    label = ET.SubElement(view, "label")
    label.text = "Inputs"
    view_as_string = ET.tostring(view, encoding="unicode")
    return _pretty_print_xml(view_as_string)


def generate_views_configuration_xml(addon_name: str) -> str:
    """
    Generates `default/data/ui/views/configuration.xml` file.
    """
    view = ET.Element(
        "view",
        attrib={
            "template": f"{addon_name}:/templates/base.html",
            "type": "html",
            "isDashboard": "False",
        },
    )
    label = ET.SubElement(view, "label")
    label.text = "Configuration"
    view_as_string = ET.tostring(view, encoding="unicode")
    return _pretty_print_xml(view_as_string)


def generate_views_redirect_xml(addon_name: str) -> str:
    """
    Generates `default/data/ui/views/redirect.xml` file.
    """
    view = ET.Element(
        "view",
        attrib={
            "template": f"{addon_name}:templates/{addon_name.lower()}_redirect.html",
            "type": "html",
            "isDashboard": "False",
        },
    )
    label = ET.SubElement(view, "label")
    label.text = "Redirect"
    view_as_string = ET.tostring(view, encoding="unicode")
    return _pretty_print_xml(view_as_string)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from io import StringIO

__all__ = [
    "RestEntityBuilder",
    "RestEndpointBuilder",
    "quote_string",
    "indent",
]

from typing import List, Sequence


class RestEntityBuilder:
    _title_template = "[{}]"
    _rh_template = """
fields{name_rh} = [
{fields}
]
model{name_rh} = RestModel(fields{name_rh}, name={name})
"""
    _disabled_field_template = """
field.RestField(
    'disabled',
    required=False,
    validator=None
)
"""

    def __init__(self, name, fields, **kwargs):
        self._name = name
        self._fields = fields
        self._conf_name = kwargs.get("conf_name")

    @property
    def name(self):
        return self._name

    @property
    def name_spec(self):
        raise NotImplementedError()

    @property
    def name_default(self):
        raise NotImplementedError()

    @property
    def name_rh(self):
        raise NotImplementedError()

    def generate_spec(self):
        title = self._title_template.format(self.name_spec)
        lines = [field.generate_spec() for field in self._fields]
        lines.insert(0, title)
        return "\n".join(lines)

    def generate_conf_with_default_values(self):
        title = self._title_template.format(self.name_spec)
        lines = [field.generate_conf_with_default_value() for field in self._fields]
        lines.insert(0, title)
        return "\n".join(lines)

    def generate_rh(self):
        fields = []
        for field in self._fields:
            field_line = field.generate_rh()
            fields.append(field_line)
        # add disabled field for data input
        entity_builder = self.__class__.__name__
        if (
            entity_builder == "DataInputEntityBuilder"
            or entity_builder == "SingleModelEntityBuilder"
            and self._conf_name
        ):
            fields.append(self._disabled_field_template)
        fields_lines = ", \n".join(fields)
        return self._rh_template.format(
            fields=indent(fields_lines),
            name_rh=self.name_rh,
            name=quote_string(self._name),
        )


class RestEndpointBuilder:
    def __init__(self, name, namespace, **kwargs):
        self._name = name
        self._namespace = namespace
        self._entities = []
        self._conf_name = (
            kwargs.get("conf_name")
            if kwargs.get("conf_name") is not None
            else self.name.lower()
        )
        if kwargs.get("rest_handler_name") is not None:
            self._rest_handler_name = kwargs.get("rest_handler_name")
        else:
            self._rest_handler_name = f"{self._namespace}_rh_{self._name}"
        self._rest_handler_module = kwargs.get("rest_handler_module")
        self._rest_handler_class = kwargs.get("rest_handler_class")

    @property
    def name(self):
        return f"{self._namespace}_{self._name}"

    @property
    def namespace(self):
        return self._namespace

    @property
    def conf_name(self):
        return self._conf_name

    @property
    def rh_name(self):
        return self._rest_handler_name

    @property
    def rh_module(self):
        return self._rest_handler_module

    @property
    def rh_class(self):
        return self._rest_handler_class

    @property
    def entities(self):
        return self._entities

    def add_entity(self, entity):
        self._entities.append(entity)

    def actions(self) -> List[str]:
        raise NotImplementedError()

    def generate_spec(self):
        specs = [entity.generate_spec() for entity in self._entities]
        return "\n\n".join(specs)

    def generate_conf_with_default_values(self):
        specs = [
            entity.generate_conf_with_default_values() for entity in self._entities
        ]
        return "\n\n".join(specs)

    def generate_rh(self) -> str:
        raise NotImplementedError()


def quote_string(value) -> str:
    """
    Quote a string
    :param value:
    :return:
    """
    if isinstance(value, str):
        return "'%s'" % value
    else:
        return value


def indent(lines: Sequence[str], spaces: int = 1) -> str:
    """
    Indent code block.

    :param lines:
    :type lines: str
    :param spaces: times of four
    :return:
    """
    string_io = StringIO(str(lines))
    indentation = spaces * 4
    prefix = " " * indentation
    lines = []
    for line in string_io:
        if line != "\n":
            line = prefix + line
        lines.append(line)
    return "".join(lines)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import re
from os import path as op

from jinja2 import Environment, FileSystemLoader

from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    arf_consts as ac,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_helper import (
    write_file,
)

logger = logging.getLogger("ucc_gen")


class AlertActionsPyBase:
    def __init__(self, addon_name: str, input_setting, package_path):
        self._addon_name = addon_name
        self._all_setting = input_setting
        self._package_path = package_path
        self._current_alert = None
        self._alert_actions_setting = input_setting[ac.MODULAR_ALERTS]
        self._ta_name = self._all_setting.get(ac.SHORT_NAME)
        self._lib_dir = self.get_python_lib_dir_name(self._ta_name)
        # nosemgrep: splunk.autoescape-disabled, python.jinja2.security.audit.autoescape-disabled.autoescape-disabled
        self._templates = Environment(
            loader=FileSystemLoader(
                op.join(op.dirname(op.realpath(__file__)), "arf_template")
            ),
            trim_blocks=True,
            lstrip_blocks=True,
            keep_trailing_newline=True,
        )

    def get_python_lib_dir_name(self, app_name):
        space_replace = re.compile(r"[^\w]+")
        return space_replace.sub("_", app_name.lower())

    def get_alert_py_name(self, helper=""):
        return self._current_alert[ac.SHORT_NAME] + helper + ".py"

    def get_alert_py_path(self):
        return op.join(self._package_path, "bin", self.get_alert_py_name())

    def get_alert_helper_py_name(self):
        return "modalert_" + self._current_alert[ac.SHORT_NAME] + "_helper.py"

    def get_alert_helper_py_path(self):
        return op.join(
            self._package_path, "bin", self._lib_dir, self.get_alert_helper_py_name()
        )


class AlertActionsPyGenerator(AlertActionsPyBase):
    def __init__(
        self,
        addon_name,
        input_setting,
        package_path,
    ):
        super().__init__(
            addon_name=addon_name,
            input_setting=input_setting,
            package_path=package_path,
        )

    def gen_py_file(self, one_alert_setting):
        self._current_alert = one_alert_setting
        self.gen_main_py_file()
        self.gen_helper_py_file()

    def gen_main_py_file(self):
        template = self._templates.get_template("alert_action.py.template")
        rendered_content = template.render(
            addon_name=self._addon_name,
            lib_name=self._lib_dir,
            mod_alert=self._current_alert,
            helper_name=op.splitext(self.get_alert_helper_py_name())[0],
        )
        logger.debug('operation="Writing file", file="%s"', self.get_alert_py_path())
        write_file(
            self.get_alert_py_name(),
            self.get_alert_py_path(),
            rendered_content,
        )

    def gen_helper_py_file(self):
        template = self._templates.get_template("alert_action_helper.py.template")
        rendered_content = template.render(
            input=self._all_setting,
            mod_alert=self._current_alert,
        )
        logger.debug('operation="Writing file", file="%s"', self.get_alert_py_path())
        write_file(
            self.get_alert_helper_py_name(),
            self.get_alert_helper_py_path(),
            rendered_content,
        )

    def handle(self):
        for alert in self._alert_actions_setting:
            logger.info(
                'operation="Generate py file", alert_action="%s"', alert[ac.SHORT_NAME]
            )
            self.gen_py_file(alert)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import os
import shutil
import stat
import subprocess
from pathlib import Path
from typing import Sequence

logger = logging.getLogger("ucc_gen")


class SplunktaucclibNotFound(Exception):
    pass


class CouldNotInstallRequirements(Exception):
    pass


def _subprocess_call(command: str, command_desc: str) -> None:
    try:
        return_code = subprocess.call(command, shell=True)
        if return_code < 0:
            logger.error(
                f"Child ({command_desc}) was terminated by signal {-return_code}"
            )
            raise CouldNotInstallRequirements
        if return_code > 0:
            logger.error(f"Command ({command_desc}) returned {return_code} status code")
            raise CouldNotInstallRequirements
    except OSError as e:
        logger.error(f"Execution ({command_desc}) failed due to {e}")
        raise CouldNotInstallRequirements from e


def _check_ucc_library_in_requirements_file(path_to_requirements: str) -> bool:
    with open(path_to_requirements) as f_reqs:
        content = f_reqs.readlines()
    for line in content:
        if "splunktaucclib" in line:
            return True
    return False


def install_python_libraries(
    source_path: str,
    ucc_lib_target: str,
    python_binary_name: str,
    includes_ui: bool = False,
):
    path_to_requirements_file = os.path.join(source_path, "lib", "requirements.txt")
    if os.path.isfile(path_to_requirements_file):
        logger.info(f"Installing requirements from {path_to_requirements_file}")
        if includes_ui:
            ucc_library_present = _check_ucc_library_in_requirements_file(
                path_to_requirements_file
            )
            if not ucc_library_present:
                raise SplunktaucclibNotFound(
                    f"splunktaucclib is not found in {path_to_requirements_file}. "
                    f"Please add it there because this add-on has UI."
                )
        if not os.path.exists(ucc_lib_target):
            os.makedirs(ucc_lib_target)
        install_libraries(
            path_to_requirements_file,
            ucc_lib_target,
            python_binary_name,
        )
        packages_to_remove = (
            "setuptools",
            "bin",
            "pip",
            "distribute",
            "wheel",
        )
        remove_package_from_installed_path(
            ucc_lib_target,
            packages_to_remove,
        )

        remove_execute_bit(ucc_lib_target)
    else:
        logger.warning(
            f"Could not find requirements file @ {path_to_requirements_file}, nothing to install"
        )


def install_libraries(
    requirements_file_path: str,
    installation_path: str,
    installer: str,
):
    """
    Upgrades `pip` version to the latest one and installs requirements to the
    specified path.
    """
    pip_update_command = f"{installer} -m pip install pip --upgrade"
    pip_install_command = (
        f"{installer} "
        f"-m pip "
        f"install "
        f'-r "{requirements_file_path}" '
        f"--no-compile "
        f"--prefer-binary "
        f"--ignore-installed "
        f"--use-deprecated=legacy-resolver "
        f'--target "{installation_path}"'
    )

    _subprocess_call(pip_update_command, "pip upgrade")
    _subprocess_call(pip_install_command, "pip install")


def remove_package_from_installed_path(
    installation_path: str, package_names: Sequence[str]
):
    p = Path(installation_path)
    for package_name in package_names:
        for o in p.glob(f"{package_name}*"):
            if o.is_dir():
                logger.info(f"  removing directory {o} from {installation_path}")
                shutil.rmtree(o)


def remove_execute_bit(installation_path: str):
    p = Path(installation_path)
    no_user_exec = ~stat.S_IEXEC
    no_group_exec = ~stat.S_IXGRP
    no_other_exec = ~stat.S_IXOTH
    no_exec = no_user_exec & no_group_exec & no_other_exec

    for o in p.rglob("*"):
        if not o.is_dir() and os.access(o, os.X_OK):
            logger.info(f"  fixing {o} execute bit")
            current_permissions = stat.S_IMODE(os.lstat(o).st_mode)
            os.chmod(o, current_permissions & no_exec)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import argparse
import sys
from typing import Optional, Sequence
import logging

from splunk_add_on_ucc_framework.commands import build
from splunk_add_on_ucc_framework.commands import init
from splunk_add_on_ucc_framework.commands import import_from_aob

logger = logging.getLogger("ucc_gen")


# This is a necessary change to have, so we don't release a breaking change.
# This class adds a default subparser if none specified. In our case it will be
# `build`subclass.
# This is a temporary change and everyone migrates to `ucc-gen build` instead of
# just using `ucc-gen` this will be removed.
# The limitation of this approach is that we can't have global options without a
# subparser being specified. Example is `--version`, the default subparser will
# be added here as well. But this is not a big deal for now, we don't have
# global options anyway.
class DefaultSubcommandArgumentParser(argparse.ArgumentParser):
    __default_subparser = None

    def set_default_subparser(self, name):
        self.__default_subparser = name

    def _parse_known_args(self, arg_strings, *args, **kwargs):
        in_args = set(arg_strings)
        d_sp = self.__default_subparser
        if d_sp is not None and not {"-h", "--help"}.intersection(in_args):
            for x in self._subparsers._actions:
                subparser_found = isinstance(
                    x, argparse._SubParsersAction
                ) and in_args.intersection(x._name_parser_map.keys())
                if subparser_found:
                    break
            else:
                logger.warning(
                    "Please use `ucc-gen build` if you want to build "
                    "an add-on, using just `ucc-gen` will be deprecated"
                )
                arg_strings = [d_sp] + arg_strings
        return super()._parse_known_args(arg_strings, *args, **kwargs)


def main(argv: Optional[Sequence[str]] = None):
    argv = argv if argv is not None else sys.argv[1:]
    parser = DefaultSubcommandArgumentParser()
    parser.set_default_subparser("build")
    subparsers = parser.add_subparsers(dest="command", description="Build an add-on")

    build_parser = subparsers.add_parser("build")
    build_parser.add_argument(
        "--source",
        type=str,
        nargs="?",
        help="Folder containing the app.manifest and app source.",
        default="package",
    )
    build_parser.add_argument(
        "--config",
        type=str,
        nargs="?",
        help="Path to configuration file, defaults to globalConfig file in parent directory of source provided.",
        default=None,
    )
    build_parser.add_argument(
        "--ta-version",
        type=str,
        help="Version of TA, default version is version specified in the "
        "package such as app.manifest, app.conf, and globalConfig file.",
        default=None,
    )
    build_parser.add_argument(
        "-o",
        "--output",
        type=str,
        help="Output path to store built add-on.",
        default=None,
    )
    build_parser.add_argument(
        "--python-binary-name",
        type=str,
        help="Python binary name to use to install requirements.",
        default="python3",
    )

    init_parser = subparsers.add_parser("init", description="Bootstrap an add-on.")
    init_parser.add_argument(
        "--addon-name",
        type=str,
        help="Add-on name.",
        required=True,
    )
    init_parser.add_argument(
        "--addon-display-name",
        type=str,
        help="Add-on display name.",
        required=True,
    )
    init_parser.add_argument(
        "--addon-input-name",
        type=str,
        help="Add-on input name.",
        required=True,
    )
    init_parser.add_argument(
        "--addon-version",
        type=str,
        help="Add-on version.",
        default="0.0.1",
    )
    init_parser.add_argument(
        "--overwrite",
        action="store_true",
        default=False,
        help="Overwrite already generated add-on folder.",
    )

    import_from_aob_parser = subparsers.add_parser(
        "import-from-aob", description="[Experimental] Import from AoB."
    )
    import_from_aob_parser.add_argument(
        "--addon-name",
        type=str,
        help="Add-on name.",
        required=True,
    )

    args = parser.parse_args(argv)
    if args.command == "build":
        build.generate(
            source=args.source,
            config_path=args.config,
            addon_version=args.ta_version,
            output_directory=args.output,
            python_binary_name=args.python_binary_name,
        )
    if args.command == "init":
        init.init(
            addon_name=args.addon_name,
            addon_display_name=args.addon_display_name,
            addon_input_name=args.addon_input_name,
            addon_version=args.addon_version,
            overwrite=args.overwrite,
        )
    if args.command == "import-from-aob":
        import_from_aob.import_from_aob(
            addon_name=args.addon_name,
        )


if __name__ == "__main__":
    raise SystemExit(main())



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import os
import sys

from cookiecutter import exceptions, main

logger = logging.getLogger("ucc_gen")


def init(
    addon_name: str,
    addon_display_name: str,
    addon_input_name: str,
    addon_version: str,
    overwrite: bool = False,
) -> str:
    try:
        generated_addon_path = main.cookiecutter(
            template=os.path.join(
                os.path.dirname(__file__),
                "init_template",
            ),
            overwrite_if_exists=overwrite,
            no_input=True,
            extra_context={
                "addon_name": addon_name,
                "addon_display_name": addon_display_name,
                "addon_input_name": addon_input_name,
                "addon_version": addon_version,
            },
        )
        logger.info(f"Generated add-on is located here {generated_addon_path}")
        logger.info(
            "LICENSE.txt and README.txt are empty, "
            "you may need to modify the content of those files. "
        )
        return generated_addon_path
    except exceptions.OutputDirExistsException:
        logger.error(
            "The location is already taken, use `--overwrite` "
            "option to overwrite the content of existing folder."
        )
        sys.exit(1)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import json
import logging
import os
from os import linesep
from os import path as op

from jinja2 import Environment, FileSystemLoader

from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    arf_consts as ac,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_helper import (
    write_file,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_merge import (
    remove_alert_from_conf_file,
)

logger = logging.getLogger("ucc_gen")


class AlertActionsConfGeneration:
    def __init__(
        self,
        input_setting,
        package_path,
    ):
        self._alert_conf_name = "alert_actions.conf"
        self._alert_spec_name = "alert_actions.conf.spec"
        self._eventtypes_conf = "eventtypes.conf"
        self._tags_conf = "tags.conf"
        self._alert_settings = input_setting[ac.MODULAR_ALERTS]
        self._package_path = package_path
        # nosemgrep: splunk.autoescape-disabled, python.jinja2.security.audit.autoescape-disabled.autoescape-disabled
        self._templates = Environment(
            loader=FileSystemLoader(
                op.join(op.dirname(op.realpath(__file__)), "arf_template")
            ),
            trim_blocks=True,
            lstrip_blocks=True,
            keep_trailing_newline=True,
        )
        self._html_fields = [ac.PARAMETERS]
        self._default_conf_settings = {
            "python.version": "python3",
            "is_custom": 1,
            "payload_format": "json",
            "icon_path": "alerticon.png",
        }

    def get_local_conf_file_path(self, conf_name):
        local_path = op.join(self._package_path, "default")
        if not op.exists(local_path):
            os.makedirs(local_path)

        return op.join(local_path, conf_name)

    def get_spec_file_path(self):
        readme_path = op.join(self._package_path, "README")
        if not op.exists(readme_path):
            os.makedirs(readme_path)
        return op.join(readme_path, self._alert_spec_name)

    def generate_conf(self):
        logger.info(
            'status="starting", operation="generate", '
            + 'object="alert_actions.conf", object_type="file"'
        )
        template = self._templates.get_template("alert_actions.conf.template")
        deny_list = frozenset(
            [
                "short_name",
                "alert_props",
                "parameters",
                "uuid",
                "code",
                "largeIcon",
                "smallIcon",
                "index",
            ]
        )
        alerts = {}
        for alert in self._alert_settings:
            alert_name = alert["short_name"]
            alerts[alert_name] = []
            for k, v in alert.items():
                if k == "active_response":
                    new_cam = {
                        sub_k: sub_v
                        for sub_k, sub_v in list(v.items())
                        if sub_k != "sourcetype" and sub_v
                    }
                    value = f"param._cam = {json.dumps(new_cam)}"
                    alerts[alert_name].append(value)
                elif k == "alert_props":
                    for pk, pv in v.items():
                        value = f"{str(pk).strip()} = {str(pv).strip()}"
                        alerts[alert_name].append(value)
                elif k not in deny_list:
                    value = f"{str(k).strip()} = {str(v).strip()}"
                    alerts[alert_name].append(value)
            for k, v in alert.items():
                if k == "parameters":
                    for param in v:
                        param_name = param["name"].strip()
                        if param.get("default_value") is not None:
                            param_default_value = str(
                                param.get("default_value")
                            ).strip()
                            alerts[alert_name].append(
                                f"param.{param_name} = {param_default_value}"
                            )
                        else:
                            alerts[alert_name].append(f"param.{param_name} = ")
        final_string = template.render(alerts=alerts)
        text = linesep.join([s.strip() for s in final_string.splitlines()])
        write_file(
            self._alert_conf_name,
            self.get_local_conf_file_path(self._alert_conf_name),
            text,
        )
        logger.info(
            'status="success", operation="generate", '
            + 'object="alert_actions.conf", object_type="file"'
        )

    def generate_eventtypes(self):
        logger.info(
            'status="starting", operation="generate", '
            + 'object="eventtypes.conf", object_type="file"'
        )
        template = self._templates.get_template("eventtypes.conf.template")
        final_string = template.render(mod_alerts=self._alert_settings)
        text = linesep.join([s.strip() for s in final_string.splitlines()])
        file_path = self.get_local_conf_file_path(self._eventtypes_conf)
        write_file(self._eventtypes_conf, file_path, text)

        # remove the stanza if not checked
        for alert in self._alert_settings:
            if alert.get("active_response") and alert["active_response"].get(
                "sourcetype"
            ):
                continue
            remove_alert_from_conf_file(alert, file_path)
        logger.info(
            'status="success", operation="generate", '
            + 'object="eventtypes.conf", object_type="file"'
        )

    def generate_tags(self):
        logger.info(
            'status="starting", operation="generate", '
            + 'object="tags.conf", object_type="file"'
        )
        template = self._templates.get_template("tags.conf.template")
        final_string = template.render(mod_alerts=self._alert_settings)
        text = linesep.join([s.strip() for s in final_string.splitlines()])
        file_path = self.get_local_conf_file_path(self._tags_conf)
        write_file(self._tags_conf, file_path, text)

        # remove the stanza if not checked
        for alert in self._alert_settings:
            if alert.get("active_response") and alert["active_response"].get(
                "sourcetype"
            ):
                continue
            remove_alert_from_conf_file(alert, file_path)
        logger.info(
            'status="success", operation="generate", '
            + 'object="tags.conf", object_type="file"'
        )

    def generate_spec(self):
        logger.info(
            'status="starting", operation="generate", '
            + 'object="alert_actions.conf.spec", object_type="file"'
        )
        template = self._templates.get_template("alert_actions.conf.spec.template")
        _router = {
            "dropdownlist": "list",
            "text": "string",
            "textarea": "string",
            "checkbox": "bool",
            "password": "password",
            "dropdownlist_splunk_search": "list",
            "radio": "list",
        }
        alerts = {}
        for alert in self._alert_settings:
            alert_name = alert["short_name"]
            alerts[alert_name] = []
            for k, v in alert.items():
                if k == "active_response":
                    alerts[alert_name].append(
                        "param._cam = <json> Active response parameters."
                    )
                elif k == "parameters":
                    for param in v:
                        format_type = _router[param["format_type"]]
                        is_required = (
                            "It's a required parameter."
                            if param.get("required") and param["required"]
                            else ""
                        )
                        param_default_value = param.get("default_value")
                        default_value = (
                            f"It's default value is {param_default_value}."
                            if param_default_value
                            else ""
                        )
                        value = (
                            f'param.{param["name"]} = <{format_type}> '
                            f'{param["label"]}. {is_required} {default_value}'
                        )
                        alerts[alert_name].append(value)
        final_string = template.render(alerts=alerts)
        text = linesep.join([s.strip() for s in final_string.splitlines()])
        write_file(self._alert_spec_name, self.get_spec_file_path(), text)
        logger.info(
            'status="success", operation="generate", '
            + 'object="alert_actions.conf.spec", object_type="file"'
        )

    def handle(self):
        self.add_default_settings()
        self.generate_conf()
        self.generate_spec()
        self.generate_eventtypes()
        self.generate_tags()

    def add_default_settings(self):
        for alert in self._alert_settings:
            if ac.ALERT_PROPS not in list(alert.keys()):
                alert[ac.ALERT_PROPS] = {}
            for k, v in list(self._default_conf_settings.items()):
                if k in list(alert[ac.ALERT_PROPS].keys()):
                    continue

                alert[ac.ALERT_PROPS][k] = v
                logger.info(
                    'status="success", operation="Add default setting", alert_name="%s", "%s"="%s"',
                    alert[ac.SHORT_NAME],
                    k,
                    v,
                )



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import json
from typing import Dict, List, Optional

APP_MANIFEST_SCHEMA_VERSION = "2.0.0"
APP_MANIFEST_SUPPORTED_DEPLOYMENTS = frozenset(
    [
        "*",
        "_standalone",
        "_distributed",
        "_search_head_clustering",
    ]
)
APP_MANIFEST_TARGET_WORKLOADS = frozenset(
    [
        "*",
        "_search_heads",
        "_indexers",
        "_forwarders",
    ]
)
APP_MANIFEST_FILE_NAME = "app.manifest"
APP_MANIFEST_WEBSITE = "https://dev.splunk.com/enterprise/reference/packagingtoolkit/pkgtoolkitappmanifest/"


class AppManifestFormatException(Exception):
    pass


class AppManifest:
    def __init__(self):
        self._manifest = None

    def get_addon_name(self) -> str:
        return self._manifest["info"]["id"]["name"]

    def get_title(self) -> str:
        return self._manifest["info"]["title"]

    def get_description(self) -> str:
        return self._manifest["info"]["description"]

    def get_license_name(self) -> str:
        return self._manifest["info"]["license"]["name"]

    def get_license_uri(self) -> str:
        return self._manifest["info"]["license"]["uri"]

    def get_authors(self) -> List[Dict[str, str]]:
        return self._manifest["info"]["author"]

    def _get_schema_version(self) -> Optional[str]:
        return self._manifest.get("schemaVersion")

    def _get_supported_deployments(self) -> Optional[List[str]]:
        return self._manifest.get("supportedDeployments")

    def _get_target_workloads(self) -> Optional[List[str]]:
        return self._manifest.get("targetWorkloads")

    @property
    def manifest(self) -> Dict:
        return self._manifest

    def read(self, content: str) -> None:
        try:
            self._manifest = json.loads(content)
        except json.JSONDecodeError:
            raise AppManifestFormatException(
                "Could not parse app.manifest, not a correct JSON file"
            )

    def update_addon_version(self, version: str) -> None:
        self._manifest["info"]["id"]["version"] = version

    def validate(self):
        schema_version = self._get_schema_version()
        if schema_version != APP_MANIFEST_SCHEMA_VERSION:
            raise AppManifestFormatException(
                f"schemaVersion should be '{APP_MANIFEST_SCHEMA_VERSION}'"
            )
        supported_deployments = self._get_supported_deployments()
        if not supported_deployments:
            raise AppManifestFormatException("supportedDeployments should be set")
        supported_deployments_set = set(supported_deployments)
        if not supported_deployments_set.issubset(APP_MANIFEST_SUPPORTED_DEPLOYMENTS):
            raise AppManifestFormatException(
                f"supportedDeployments should only have values from '{APP_MANIFEST_SUPPORTED_DEPLOYMENTS}'"
            )
        target_workloads = self._get_target_workloads()
        if not target_workloads:
            raise AppManifestFormatException("targetWorkloads should be set")
        target_workloads_set = set(target_workloads)
        if not target_workloads_set.issubset(APP_MANIFEST_TARGET_WORKLOADS):
            raise AppManifestFormatException(
                f"targetWorkloads should only have values from '{APP_MANIFEST_TARGET_WORKLOADS}'"
            )

    def __str__(self) -> str:
        return json.dumps(self._manifest, indent=4, sort_keys=True)



import json
import logging
import sys
import traceback

import import_declare_test
from solnlib import conf_manager, log
from splunklib import modularinput as smi

ADDON_NAME = "{{cookiecutter.addon_name}}"


def logger_for_input(input_name: str) -> logging.Logger:
    return log.Logs().get_logger(f"{ADDON_NAME.lower()}_{input_name}")


def get_account_api_key(session_key: str, account_name: str):
    cfm = conf_manager.ConfManager(
        session_key,
        ADDON_NAME,
        realm=f"__REST_CREDENTIAL__#{ADDON_NAME}#configs/conf-{{cookiecutter.addon_name}}_account",
    )
    account_conf_file = cfm.get_conf("{{cookiecutter.addon_name}}_account")
    return account_conf_file.get(account_name).get("username")


def get_data_from_api(logger: logging.Logger, api_key: str):
    logger.info("Getting data from an external API")
    dummy_data = [
        {
            "line1": "hello",
        },
        {
            "line2": "world",
        },
    ]
    return dummy_data


class Input(smi.Script):
    def __init__(self):
        super().__init__()

    def get_scheme(self):
        scheme = smi.Scheme("{{cookiecutter.addon_input_name}}")
        scheme.description = "{{cookiecutter.addon_input_name}} input"
        scheme.use_external_validation = True
        scheme.streaming_mode_xml = True
        scheme.use_single_instance = False
        scheme.add_argument(
            smi.Argument(
                "name", title="Name", description="Name", required_on_create=True
            )
        )
        return scheme

    def validate_input(self, definition: smi.ValidationDefinition):
        return

    def stream_events(self, inputs: smi.InputDefinition, event_writer: smi.EventWriter):
        # inputs.inputs is a Python dictionary object like:
        # {
        #   "{{cookiecutter.addon_input_name}}://<input_name>": {
        #     "account": "<account_name>",
        #     "disabled": "0",
        #     "host": "$decideOnStartup",
        #     "index": "<index_name>",
        #     "interval": "<interval_value>",
        #     "python.version": "python3",
        #   },
        # }
        for input_name, input_item in inputs.inputs.items():
            normalized_input_name = input_name.split("/")[-1]
            logger = logger_for_input(normalized_input_name)
            try:
                session_key = self._input_definition.metadata["session_key"]
                log_level = conf_manager.get_log_level(
                    logger=logger,
                    session_key=session_key,
                    app_name=ADDON_NAME,
                    conf_name=f"{ADDON_NAME}_settings",
                )
                logger.setLevel(log_level)
                log.modular_input_start(logger, normalized_input_name)
                api_key = get_account_api_key(session_key, input_item.get("account"))
                data = get_data_from_api(logger, api_key)
                sourcetype = "dummy-data"
                for line in data:
                    event_writer.write_event(
                        smi.Event(
                            data=json.dumps(line, ensure_ascii=False, default=str),
                            index=input_item.get("index"),
                            sourcetype=sourcetype,
                        )
                    )
                log.events_ingested(
                    logger, normalized_input_name, sourcetype, len(data)
                )
                log.modular_input_end(logger, normalized_input_name)
            except Exception as e:
                logger.error(
                    f"Exception raised while ingesting data for "
                    f"{{cookiecutter.addon_input_name}}: {e}. Traceback: "
                    f"{traceback.format_exc()}"
                )


if __name__ == "__main__":
    exit_code = Input().run(sys.argv)
    sys.exit(exit_code)



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging
import os
from os import linesep
from os import path as op
from re import search
from typing import Dict, Any

from jinja2 import Environment, FileSystemLoader

from splunk_add_on_ucc_framework.commands.modular_alert_builder import (
    arf_consts as ac,
)
from splunk_add_on_ucc_framework.commands.modular_alert_builder.alert_actions_helper import (
    write_file,
)

logger = logging.getLogger("ucc_gen")


class AlertHtmlGenerator:
    def __init__(
        self,
        input_setting,
        package_path,
    ):
        self._all_setting = input_setting
        self._package_path = package_path
        # nosemgrep: splunk.autoescape-disabled, python.jinja2.security.audit.autoescape-disabled.autoescape-disabled
        self._templates = Environment(
            loader=FileSystemLoader(
                [
                    op.join(
                        op.dirname(op.realpath(__file__)),
                        "arf_template",
                        "default_html_theme",
                    ),
                    op.join(op.dirname(op.realpath(__file__)), "arf_template"),
                ]
            ),
            trim_blocks=True,
            lstrip_blocks=True,
            keep_trailing_newline=True,
        )
        self._alert_actions_setting = input_setting[ac.MODULAR_ALERTS]
        self._html_template = self._templates.get_template("mod_alert.html.template")
        self._html_home = "default.html"

    def get_alert_html_name(self, alert: Dict[str, Any]) -> str:
        return alert[ac.SHORT_NAME] + ".html"

    def get_alert_html_path(self, alert: Dict[str, Any]) -> str:
        html_path = op.join(self._package_path, "default", "data", "ui", "alerts")
        if not op.exists(html_path):
            os.makedirs(html_path)

        return op.join(html_path, self.get_alert_html_name(alert))

    def handle_alert(self, alert: Dict[str, Any]) -> None:
        rendered = self._html_template.render(
            mod_alert=alert, home_page=self._html_home
        )
        text = linesep.join(
            [s for s in rendered.splitlines() if not search(r"^\s*$", s)]
        )

        logger.info(f"Creating file @ {self.get_alert_html_path(alert)}")

        write_file(
            self.get_alert_html_name(alert),
            self.get_alert_html_path(alert),
            text,
        )

    def handle(self):
        logger.info("Started generating alert actions HTML files")
        for alert in self._alert_actions_setting:
            alert_short_name = alert["short_name"]
            logger.info(f"Generating HTML file for '{alert_short_name}'")
            self.handle_alert(alert)
        logger.info("Finished generating alert actions HTML files")



#
# Copyright 2021 Splunk Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import List

from splunk_add_on_ucc_framework.commands.rest_builder.endpoint.base import indent
from splunk_add_on_ucc_framework.commands.rest_builder.endpoint.single_model import (
    RestEndpointBuilder,
    RestEntityBuilder,
)


class MultipleModelEntityBuilder(RestEntityBuilder):
    @property
    def name_spec(self):
        return self.name

    @property
    def name_default(self):
        return self.name

    @property
    def name_rh(self):
        return "_" + self._name


class MultipleModelEndpointBuilder(RestEndpointBuilder):
    _rh_template = """
import import_declare_test

from splunktaucclib.rest_handler.endpoint import (
    field,
    validator,
    RestModel,
    MultipleModel,
)
from splunktaucclib.rest_handler import admin_external, util
from {handler_module} import {handler_class}
import logging

util.remove_http_proxy_env_vars()

{entities}

endpoint = MultipleModel(
    '{conf_name}',
    models=[
{models}
    ],
)


if __name__ == '__main__':
    logging.getLogger().addHandler(logging.NullHandler())
    admin_external.handle(
        endpoint,
        handler={handler_class},
    )
"""

    def actions(self) -> List[str]:
        return ["edit", "list"]

    def generate_rh(self) -> str:
        entities = [entity.generate_rh() for entity in self._entities]
        models = ["model" + entity.name_rh for entity in self._entities]
        models_lines = ", \n".join(models)
        return self._rh_template.format(
            handler_module=self.rh_module,
            handler_class=self.rh_class,
            entities="\n".join(entities),
            models=indent(models_lines, 2),
            conf_name=self.conf_name,
        )
